/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * UTP Cable Tester - Vlad Paunescu 332CB
 * Digital I/O
 *********************************************************************************/

#include <stdio.h>
#include <avr/io.h>
#define F_CPU 16000000UL
#include <util/delay.h>


/********************************
 *	DEFINE-uri pentru tester	*
 *******************************/
#define DSEND				DDRA
#define DRECEIVE			DDRC
#define ISEND				PINA
#define IRECEIVE			PINC
#define OSEND				PORTA
#define ORECEIVE			PORTC

#define DDECISION DDRD
#define IDECISION PIND
#define ODECISION PORTD

# define STRAIGHT_LED 0
# define CROSS_LED 1
# define ROLL_LED 2
# define BAD_LED 3

// definirea tipurilor de cablu
int STRAIGHT[8]= {1, 2, 3, 4, 5, 6, 7, 8 };
int CROSS[8]= {3, 6, 1, 4, 5, 2, 7, 8 };
int ROLLOVER[8]=	{3, 6, 1, 7, 8, 2, 4, 5 };

// vectorul in care se face citirea
int TEST[8]= {0, 0, 0, 0, 0, 0, 0, 0 };

//default status este 0
int status = 0;

int getActiveBit(int x) {
//determina bitul setat pe 1
	int i;
	
	for(i=0; i<8; i++){
		if(bit_is_set(x, i)){
			return i + 1;
		}
	}
	return 0;
}

// compara cei doi vectori 
int compare(int *n, int *m) {
	int i;
	
	for(i=0; i<8; i++){
		if(n[i]!=m[i]){
			return 0;
		}
	}
	return 1;
}

//seteaza statusul
void setStatus(int value) {
	status = value;
}

//returneaza statusul
int getStatus() {
	return status;
}

// functie care seteaza decizia
void setDecision(int pin){
    ODECISION = (1<<pin);
}

// functe care sterge decizia
void clearDecision(){
    ODECISION=0x00;
}

// functie pentru determinare cablu si afisare mesaj corespunzator
void determineCable() {	
		
	if(compare(TEST, STRAIGHT)) {
		setDecision(STRAIGHT_LED);
    }
	else if(compare(TEST, CROSS)) {
		setDecision(CROSS_LED);
    }
	else if(compare(TEST, ROLLOVER)) {
		setDecision(ROLL_LED);
    }
	else {
        setDecision(BAD_LED);
    }
}

//verifica daca este inserat cablul
int isCableInserted() {
	int i, r;
	for(i=0; i<8; i++) {
		OSEND = 1<<i;
		r = IRECEIVE;
		
		if(getActiveBit(r) != 0) {
			OSEND=0x00;
			return 1;
		}
	}
	
	return 0;
}

int main() {	
	int i;
	int r;
	
	DSEND = 0xFF;
	DRECEIVE = 0x00;
    DDECISION = 0xFF;
	
	do {       
		if(isCableInserted()) {
            
             clearDecision();
			
            // testez cei 8 pini trimitand pe fiecare
			for(i=0; i<8; i++) {			
				
				OSEND = 1<<i;
				_delay_ms(1000);
				
				// si vad pe ce prin am primit
				r = IRECEIVE;
				
				// memorez in vector
				TEST[i] = getActiveBit(r);				
				
				//daca avem conexiune pe firul curent
				if(TEST[i] != 0) {				
					
					//daca cel putin un fir este conectat => avem cablu conectat
					if(getStatus() == 0)
						setStatus(1);
						
				//altfel marcam pe LCD ca nu avem conexiune
				} else {
                    setDecision(BAD_LED);
                    setStatus(0);
                    break;
                }				
				_delay_ms(200);
			}
			
			OSEND=0x00;			
			
            // daca am determinat cablul
			if(getStatus() == 1) {
				determineCable();
				_delay_ms(3000);
				setStatus(0);
			}
		}
        else {
            // daca nu avem cablu inserat
            setDecision(BAD_LED);
        }      
	} while(1);
	
	return 0;
	
}

